@@ -10,6 +10,7 @@ from urh.ainterpretation import Wavelet
 from urh.cythonext import auto_interpretation as c_auto_interpretation
 from urh.cythonext import signal_functions
 from urh.cythonext import util
+from urh.signalprocessing.IQArray import IQArray
 
 
 def max_without_outliers(data: np.ndarray, z=3):
@@ -47,6 +48,16 @@ def get_most_frequent_value(values: list):
     return result
 
 
+def most_common(values: list):
+    """
+    Return the most common value in a list. In case of ties, return the value that appears first in list
+    :param values:
+    :return:
+    """
+    counter = Counter(values)
+    return max(values, key=counter.get)
+
+
 def detect_noise_level(magnitudes):
     if len(magnitudes) <= 3:
         return 0
@@ -59,7 +70,7 @@ def detect_noise_level(magnitudes):
 
     mean_values = np.fromiter((np.mean(chunk) for chunk in chunks), dtype=np.float32, count=len(chunks))
     minimum, maximum = util.minmax(mean_values)
-    if minimum / maximum > 0.9:
+    if maximum == 0 or minimum / maximum > 0.9:
         # Mean values are very close to each other, so there is probably no noise in the signal
         return 0
 
@@ -91,7 +102,7 @@ def merge_message_segments_for_ook(segments: list):
         return segments
 
     result = []
-    # Get a array of pauses for comparision
+    # Get a array of pauses for comparison
     pauses = np.fromiter(
         (segments[i + 1][0] - segments[i][1] for i in range(len(segments) - 1)),
         count=len(segments) - 1,
@@ -148,7 +159,7 @@ def detect_modulation(data: np.ndarray, wavelet_scale=4, median_filter_order=11)
     var_filtered_mag = np.var(c_auto_interpretation.median_filter(mag_wavlt, k=median_filter_order))
     var_filtered_norm_mag = np.var(c_auto_interpretation.median_filter(norm_mag_wavlt, k=median_filter_order))
 
-    if all(v < 0.1 for v in (var_mag, var_norm_mag, var_filtered_mag, var_filtered_norm_mag)):
+    if all(v < 0.15 for v in (var_mag, var_norm_mag, var_filtered_mag, var_filtered_norm_mag)):
         return "OOK"
 
     if var_mag > 1.5 * var_norm_mag:
@@ -175,17 +186,20 @@ def detect_modulation(data: np.ndarray, wavelet_scale=4, median_filter_order=11)
                 return "OOK"
 
 
-def detect_modulation_for_messages(signal: np.ndarray, message_indices: list) -> str:
+def detect_modulation_for_messages(signal: IQArray, message_indices: list) -> str:
+    max_messages = 100
+
     modulations_for_messages = []
-    for start, end in message_indices:
-        mod = detect_modulation(signal[start:end])
+    complex = signal.as_complex64()
+    for start, end in message_indices[0:max_messages]:
+        mod = detect_modulation(complex[start:end])
         if mod is not None:
             modulations_for_messages.append(mod)
 
     if len(modulations_for_messages) == 0:
         return None
 
-    return max(set(modulations_for_messages), key=modulations_for_messages.count)
+    return most_common(modulations_for_messages)
 
 
 def detect_center(rectangular_signal: np.ndarray, max_size=None):
@@ -204,12 +218,16 @@ def detect_center(rectangular_signal: np.ndarray, max_size=None):
     # If a signal has low variance we need to be more accurate at center detection
     hist_step = float(np.var(rect))
 
-    y, x = np.histogram(rect, bins=np.arange(hist_min, hist_max + hist_step, hist_step))
+    try:
+        y, x = np.histogram(rect, bins=np.arange(hist_min, hist_max + hist_step, hist_step))
+    except (ZeroDivisionError, ValueError):
+        # For a segment with zero variance (constant line) it is not possible to find a center
+        return None
 
     num_values = 2
     most_common_levels = []
 
-    window_size = max(2, int(0.05*len(y)))
+    window_size = max(2, int(0.05*len(y)) + 1)
 
     def get_elem(arr, index: int, default):
         if 0 <= index < len(arr):
@@ -221,7 +239,7 @@ def detect_center(rectangular_signal: np.ndarray, max_size=None):
         # check if we have a local maximum in histogram, if yes, append the value
         if all(y[index] > get_elem(y, index+i, 0) and
                y[index] > get_elem(y, index-i, 0)
-               for i in range(1, window_size+1)):
+               for i in range(1, window_size)):
             most_common_levels.append(x[index])
 
         if len(most_common_levels) == num_values:
@@ -262,28 +280,7 @@ def merge_plateau_lengths(plateau_lengths, tolerance=None) -> list:
     if tolerance == 0 or tolerance is None:
         return plateau_lengths
 
-    result = []
-    if len(plateau_lengths) == 0:
-        return result
-
-    if plateau_lengths[0] <= tolerance:
-        result.append(0)
-
-    i = 0
-    while i < len(plateau_lengths):
-        if plateau_lengths[i] <= tolerance:
-            # Look forward to see if we need to merge a larger window e.g. for 67, 1, 10, 1, 21
-            n = 2
-            while i + n < len(plateau_lengths) and plateau_lengths[i + n] <= tolerance:
-                n += 2
-
-            result[-1] = sum(plateau_lengths[max(i - 1, 0):i + n])
-            i += n
-        else:
-            result.append(plateau_lengths[i])
-            i += 1
-
-    return result
+    return c_auto_interpretation.merge_plateaus(plateau_lengths, tolerance, max_count=10000)
 
 
 def round_plateau_lengths(plateau_lengths: list):
@@ -325,8 +322,8 @@ def get_bit_length_from_plateau_lengths(merged_plateau_lengths) -> int:
         return int(merged_plateau_lengths[0])
 
     round_plateau_lengths(merged_plateau_lengths)
+    histogram = c_auto_interpretation.get_threshold_divisor_histogram(merged_plateau_lengths)
 
-    histogram = c_auto_interpretation.get_threshold_divisor_histogram(np.array(merged_plateau_lengths, dtype=np.uint64))
     if len(histogram) == 0:
         return 0
     else:
@@ -344,8 +341,11 @@ def get_bit_length_from_plateau_lengths(merged_plateau_lengths) -> int:
         return int(result)
 
 
-def estimate(signal: np.ndarray, noise: float = None, modulation: str = None) -> dict:
-    magnitudes = np.abs(signal)
+def estimate(iq_array: IQArray, noise: float = None, modulation: str = None) -> dict:
+    if isinstance(iq_array, np.ndarray):
+        iq_array = IQArray(iq_array)
+
+    magnitudes = iq_array.magnitudes
     # find noise threshold
     noise = detect_noise_level(magnitudes) if noise is None else noise
 
@@ -353,7 +353,7 @@ def estimate(signal: np.ndarray, noise: float = None, modulation: str = None) ->
     message_indices = segment_messages_from_magnitudes(magnitudes, noise_threshold=noise)
 
     # detect modulation
-    modulation = detect_modulation_for_messages(signal, message_indices) if modulation is None else modulation
+    modulation = detect_modulation_for_messages(iq_array, message_indices) if modulation is None else modulation
     if modulation is None:
         return None
 
@@ -361,11 +361,11 @@ def estimate(signal: np.ndarray, noise: float = None, modulation: str = None) ->
         message_indices = merge_message_segments_for_ook(message_indices)
 
     if modulation == "OOK" or modulation == "ASK":
-        data = signal_functions.afp_demod(signal, noise, 0)
+        data = signal_functions.afp_demod(iq_array.data, noise, "ASK", 2)
     elif modulation == "FSK":
-        data = signal_functions.afp_demod(signal, noise, 1)
+        data = signal_functions.afp_demod(iq_array.data, noise, "FSK", 2)
     elif modulation == "PSK":
-        data = signal_functions.afp_demod(signal, noise, 2)
+        data = signal_functions.afp_demod(iq_array.data, noise, "PSK", 2)
     else:
         raise ValueError("Unsupported Modulation")
 
